const express = require('express');

const glossEnrollmentRatioApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const multiQuery = require(`${libs}/middlewares/multiQuery`);

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

const addMissing = require(`${libs}/middlewares/addMissing`);

const config = require(`${libs}/config`);

const download = require(`${libs}/middlewares/downloadDatabase`);

const passport = require('passport');

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

glossEnrollmentRatioApp.use(cache('15 day'));

// Complete range of the enrollments dataset.
// Returns a tuple of start and ending years of the complete enrollments dataset.
glossEnrollmentRatioApp.get('/year_range', (req, res, next) => {
    req.sql.from('pnad')
    .field('DISTINCT pnad.ano_censo', 'year');
    next();
}, query, (req, res, next) => {
    req.oldResult = req.result;

    req.sql = squel.select();

    req.sql.from('matricula')
    .field('DISTINCT matricula.ano_censo', 'year');
    next();
}, query, (req, res, next) => {
    let distinct_years = [];
    let new_result = [];
    for (let i = 0; i < req.oldResult.length; i++) {
        for (let j = 0; j < req.result.length; j++) {
            if(req.oldResult[i].year == req.result[j].year) {
                distinct_years.push(req.oldResult[i]);
            }
        }
    }
    new_result.push({start_year: distinct_years[distinct_years.length -1].year, end_year: distinct_years[0].year});
    req.result = new_result;
    next();
}, response('range'));

glossEnrollmentRatioApp.get('/years', (req, res, next) => {
    req.sql.from('pnad')
    .field('DISTINCT pnad.ano_censo', 'year');
    next();
}, query, (req, res, next) => {
    req.oldResult = req.result;

    req.sql = squel.select();

    req.sql.from('matricula')
    .field('DISTINCT matricula.ano_censo', 'year');
    next();
}, query, (req, res, next) => {
    let new_result = []
    for (let i = 0; i < req.oldResult.length; i++) {
        for (let j = 0; j < req.result.length; j++) {
            if(req.oldResult[i].year == req.result[j].year) {
                new_result.push(req.oldResult[i]);
            }
        }
    }
    req.result = new_result;
    next();
}, response('years'));

glossEnrollmentRatioApp.get('/source', (req, res, next) => {
    req.sql.from('fonte')
    .field('fonte', 'source')
    .where('tabela = \'pnad\'');
    next();
}, query, response('source'));

glossEnrollmentRatioApp.get('/location', (req, res, next) => {
  req.result = [
    {id: 1, name: 'Urbana'},
    {id: 2, name: 'Rural'}
  ];
  next();
}, response('location'));

glossEnrollmentRatioApp.get('/ethnic_group_pnad', (req, res, next) => {
  req.result = [
    {id: 0, name: 'Indígena'},
    {id: 1, name: 'Branca e amarela'},
    {id: 2, name: 'Preta e parda'},
    {id: 9, name: 'Sem declaração'}
  ];
  next();
}, response('ethnic_group_pnad'));

glossEnrollmentRatioApp.get('/gender', (req, res, next) => {
    req.result = [
        {id: 2, name: 'Masculino'},
        {id: 4, name: 'Feminino'}
    ];
    next();
}, response('gender'));


glossEnrollmentRatioApp.get('/fifth_household_income', (req, res, next) => {
    req.result = [
		    {id: 1, name: '20% menores'},
        {id: 2, name: '2o quinto'},
        {id: 3, name: '3o quinto'},
        {id: 4, name: '4o quinto'},
		    {id: 5, name: '20% maiores'},
        {id: -1, name: 'Sem declaração'}
	];
    next();
},response('fifth_household_income'));

glossEnrollmentRatioApp.get('/extremes_household_income', (req, res, next) => {
    req.result = [
		    {id: 1, name: '10% menores'},
        {id: 2, name: '10% maiores'},
        {id: -1, name: 'Sem declaração'}
	  ];
    next();
}, response('extremes_household_income'));

glossEnrollmentRatioApp.get('/education_level_short', (req, res, next) => {
    req.result = [
        {id: null, name: 'Não classificada'},
        {id: 1, name: 'Creche'},
        {id: 2, name: 'Pré-Escola'},
        {id: 4, name: 'Ensino Fundamental - anos iniciais'},
        {id: 5, name: 'Ensino Fundamental - anos finais'},
        {id: 6, name: 'Ensino Médio'}
    ];
    next();
}, response('education_level_short'));


rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'region',
    table: 'regiao',
    tableField: 'nome',
    resultField: 'region_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'regiao_id',
        foreignTable: 'pnad'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: 'pnad'
    }
}).addValue({
    name: 'ethnic_group_pnad',
    table: 'pnad',
    tableField: 'cor_raca_id',
    resultField: 'ethnic_group_pnad_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cor_raca_id'
    }
}).addValue({
    name: 'min_year',
    table: '@',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        table: '@',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: '@',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        table: '@',
        field: 'ano_censo'
    }
}).addValue({
    name: 'gender',
    table: 'pnad',
    tableField: 'sexo',
    resultField: 'gender_pnad_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'sexo'
    }
}).addValue({
    name: 'location',
    table: '@',
    tableField: 'localizacao_id',
    resultField: 'location_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'localizacao_id'
    }
}).addValue({
    name: 'education_level_short',
    table: 'matricula',
    tableField: 'etapa_resumida',
    resultField: 'education_level_short_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'etapa_resumida'
    }
}).addValue({
    name: 'extremes_household_income',
    table: 'pnad',
    tableField: 'extremos_nivel_rendimento',
    resultField: 'extremes_household_income_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'extremos_nivel_rendimento'
    }
}).addValue({
    name: 'fifth_household_income',
    table: 'pnad',
    tableField: 'quintil_nivel_rendimento',
    resultField: 'fifth_household_income_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'quintil_nivel_rendimento'
    }
});


function matchQueries(queryTotal, queryPartial) {
    let match = [];
    queryTotal.forEach((result) => {
        let newObj = {};
        let keys = Object.keys(result);
        keys.forEach((key) => {
            newObj[key] = result[key];
        });
        let index = keys.indexOf('total');
        if(index > -1) keys.splice(index, 1);
        let objMatch = null;

        for(let i = 0; i < queryPartial.length; ++i) {
            let partial = queryPartial[i];
            let foundMatch = true;
            for(let j = 0; j < keys.length; ++j) {
                let key = keys[j];
                if(partial[key] !== result[key]) {
                    foundMatch = false;
                    break;
                }
            }
            if(foundMatch) {
                objMatch = partial;
                break;
            }
        }

        if(objMatch) {
            newObj.denominator = result.total;
            newObj.partial = objMatch.total;
            newObj.total = (objMatch.total / result.total) * 100;
            match.push(newObj);
        }
    });

    return match;
}

glossEnrollmentRatioApp.get('/', rqf.parse(),(req, res, next) => {
    req.numerator = {};
    req.denominator = {};
    let glossEnrollmentRatioApp = {};

    log.debug(req.sql.toParam());
    req.sql.from('matricula')
    .field('sum(matricula.etapa_resumida)', 'total')
    .field('matricula.ano_censo', 'year')
    .group('matricula.ano_censo')
    .order('matricula.ano_censo')

    next();
}, rqf.build(), query, (req, res, next) => {
    req.numerator = req.result;
    req.resetSql();
    log.debug(req.sql.toParam());
    req.sql.field('sum(pnad.idade_31_03)', 'total')
    .field('pnad.ano_censo','year')
    .from('pnad')
    .group('pnad.ano_censo')
    .order('pnad.ano_censo')
    if ("education_level_short" in req.filter) {
        if (req.filter.education_level_short == 1) {
          req.sql.where('pnad.faixa_etaria_31_03 = 1')
        }
        else if (req.filter.education_level_short == 2) {
          req.sql.where('pnad.faixa_etaria_31_03 = 2')
        }
        else if (req.filter.education_level_short == 4) {
          req.sql.where('pnad.faixa_etaria_31_03 = 3')
        }
        else if (req.filter.education_level_short == 5) {
          req.sql.where('pnad.faixa_etaria_31_03 = 4')
        }
        else if (req.filter.education_level_short == 6) {
          req.sql.where('pnad.faixa_etaria_31_03 = 5')
        }
    }

    next();
}, rqf.parse(), (req, res, next) => {
   if ("education_level_short" in req.filter) {
      delete req.filter.education_level_short;
   }
   next();
}, rqf.build(), query, (req, res, next) => {
    req.denominator = req.result;
    log.debug(req.numerator);
    log.debug(req.denominator);

     for (let i = 0; i < req.numerator.length; i++) {
         req.numerator[i].total = req.numerator[i].total/req.denominator[i].total;
     }
    req.result = req.numerator;
    next();
}, response('glossEnrollmentRatio'));

glossEnrollmentRatioApp.get('/download', passport.authenticate('bearer', { session: false }), rqf.parse(), rqf.build(), download('pnad', 'mapping_pnad'));

module.exports = glossEnrollmentRatioApp;
